from icarus.http.solr_http import http_get_json_response
from icarus.query.query_model import SelectRangeSubQuery, FilterRangeSubQuery, SelectSubQuery, FilterSubQuery, OR, AND, ASCENDING
from icarus.query.select_request_builder import build_select_request
from icarus.result.select_response_handler import build_select_result

__author__ = 'shillaker'


# Internal class used to specify queries
class SearchQuerySpec():
    def __init__(self):
        self.q_sub_queries = list()
        self.fq_sub_queries = list()
        self.sort_field = None
        self.sort_order = None
        self.result_fields = None
        self.facet_fields = None
        self.facet_ranges = list()


# Wrapper class for a Solr query
class SearchQuery():
    def __init__(self, interface):
        self.interface = interface
        self.spec = SearchQuerySpec()

    # Adds a Q subquery
    def q(self, external_bool, field, value, internal_bool=OR):
        self.spec.q_sub_queries.append(SelectSubQuery(field, value, external_bool, internal_bool))
        return self

    # ANDs a Q subquery
    def q_and(self, field, value, internal_bool=OR):
        self.q(AND, field, value, internal_bool)
        return self

    # ORs a Q subquery
    def q_or(self, field, value, internal_bool=OR):
        self.q(OR, field, value, internal_bool)
        return self

    # Adds a Q range subquery
    def q_range(self, external_bool, field, range_bounds, exclusive=False):
        self.spec.q_sub_queries.append(SelectRangeSubQuery(field, range_bounds, external_bool, exclusive))
        return self

    # ANDs a Q range subquery
    def q_range_and(self, field, range_bounds, exclusive=False):
        self.spec.q_sub_queries.append(SelectRangeSubQuery(field, range_bounds, AND, exclusive))
        return self

    # ORs a Q range subquery
    def q_range_or(self, field, range_bounds, exclusive=False):
        self.spec.q_sub_queries.append(SelectRangeSubQuery(field, range_bounds, OR, exclusive))
        return self

    # Adds a FQ subquery
    def fq(self, external, field, value, internal_bool=OR):
        self.spec.fq_sub_queries.append(FilterSubQuery(field, value, external, internal_bool))
        return self

    # ANDs a FQ subquery
    def fq_and(self, field, value, internal_bool=OR):
        self.fq(AND, field, value, internal_bool)
        return self

    # ORs a FQ subquery
    def fq_or(self, field, value, internal_bool=OR):
        self.fq(OR, field, value, internal_bool)
        return self

    # Adds a FQ range subquery
    def fq_range(self, external_bool, field, range_bounds, exclusive=False):
        self.spec.fq_sub_queries.append(FilterRangeSubQuery(field, range_bounds, external_bool, exclusive))
        return self

    # ANDs a FQ range subquery
    def fq_range_and(self, field, range_bounds, exclusive=False):
        self.fq_range(AND, field, range_bounds, exclusive)
        return self

    # ORs a FQ range subquery
    def fq_range_or(self, field, range_bounds, exclusive=False):
        self.fq_range(OR, field, range_bounds, exclusive)
        return self

    # Add sort to normal query
    def sort(self, field, order=ASCENDING):
        self.spec.sort_field = field
        self.spec.sort_order = order
        return self

    # Add result fields
    def result_fields(self, fields):
        self.spec.result_fields = fields
        return self

    # Add result fields
    def facet_fields(self, facet_fields):
        self.spec.facet_fields = facet_fields
        return self

    # Add a facet range field
    def add_facet_range(self, facet_range):
        self.spec.facet_ranges.append(facet_range)
        return self

    # Execute the query
    def execute(self, start_idx, n_items=10):
        request_string = build_select_request(self, start_idx, n_items)
        response = http_get_json_response(self.interface, request_string)
        result = build_select_result(response)

        return result
